VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DocProperties"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'*********************************************************************************************
'
' OLEDocumentProperties class
'
' This class reads and writes properties to OLE documents
' including Office documents.
'
'*********************************************************************************************
'
' Author: Eduardo Morcillo
' E-Mail: edanmo@geocities.com
' Web Page: http://www.geocities.com/SiliconValley/Foothills/9940
'
' Created: 07/31/1999
' Last Updated:
'
'        08/03/1999. * pvRead* and pvWrite* were removed.
'                      Properties now use GetPropertyByName
'                      and SetPropertyByName.
'                    * GetPropertyByName and SetPropertyByName
'                      Name parameter was changed to
'                      Variant to pass the property ID
'                      in numeric format.
'                    * Added support for strings array
'                      properties.
'                    * Added BindToStorage method
'                    * Load was renamed to BindToFile
'
'        07/31/1999. * Added User properties support.
'                    * GetPropertyByName bug (always read
'                      PIDSI_CHARCOUNT) corrected.
'
'*********************************************************************************************

Option Explicit

Dim m_Storage As IStorage
Dim m_PropSet As IPropertySetStorage
    
Enum odpErrors
    odpInvalidFile = vbObjectError
    odpFileIsClosed
    odpPropertySetDoesntExist
End Enum

Const odpFileIsCloseStr = "The file is closed."
Const odpPropertySetDoesntExistStr = "The property set does not exist."

Enum odpPropertySets
    odpSummary
    odpDocSummary
    odpUserProperties
End Enum

Public Property Get Author() As String
    
    Author = GetPropertyByName(odpSummary, PIDSI_AUTHOR)

End Property
Public Property Let Author(ByVal New_Value As String)
    
    SetPropertyByName odpSummary, PIDSI_AUTHOR, New_Value

End Property



Public Sub CloseFile()

    Set m_PropSet = Nothing
    Set m_Storage = Nothing
    
End Sub

Public Sub EnumProperties(ByVal PropSet As odpPropertySets, PropNames() As Variant)
Dim Enm As IEnumSTATPROPSTG, IPS As IPropertyStorage
Dim Cnt As Long, SPS As STATPROPSTG

    On Error Resume Next
    
    Select Case PropSet
        Case odpSummary
            Set IPS = m_PropSet.Open(FMTID_SummaryInformation, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
        Case odpDocSummary
            Set IPS = m_PropSet.Open(FMTID_DocSummaryInformation, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
        Case odpUserProperties
            Set IPS = m_PropSet.Open(FMTID_UserProperties, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
    End Select
    
    If IPS Is Nothing Then Exit Sub
    
    Set Enm = IPS.Enum

    Erase PropNames
    
    Do While Enm.Next(, SPS) <> 0
        
        ReDim Preserve PropNames(0 To Cnt)
        
        If SPS.lpwstrName = 0 Then
            PropNames(Cnt) = SPS.PropID
        Else
            PropNames(Cnt) = Ptr2Str(SPS.lpwstrName, True, True)
        End If
        
        Cnt = Cnt + 1
        
    Loop
    
    Set Enm = Nothing
    Set IPS = Nothing
    
End Sub


Public Function GetPropertyByName(ByVal PropSet As odpPropertySets, ByVal Name As Variant) As Variant
Dim PROP As PROPSPEC, Value As Variant, PropName As String
Dim IPS As IPropertyStorage, VT As Integer

    If m_PropSet Is Nothing Then Err.Raise odpFileIsClosed, , odpFileIsCloseStr
    
    On Error Resume Next
    
    ' Open the desired property set
    Select Case PropSet
        Case odpSummary
            Set IPS = m_PropSet.Open(FMTID_SummaryInformation, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
        Case odpDocSummary
            Set IPS = m_PropSet.Open(FMTID_DocSummaryInformation, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
        Case odpUserProperties
            Set IPS = m_PropSet.Open(FMTID_UserProperties, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
    End Select
    
    If IPS Is Nothing Then
        On Error GoTo 0
        Err.Raise odpPropertySetDoesntExist, , odpPropertySetDoesntExistStr
    End If
    
    ' Set property ID/Name
    If IsNumeric(Name) Then
        
        PROP.ulKind = PRSPEC_PROPID
        PROP.ID_or_LPWSTR = CLng(Name)
        
    Else
    
        PropName = CStr(Name)
        
        PROP.ulKind = PRSPEC_LPWSTR
        PROP.ID_or_LPWSTR = StrPtr(PropName)
        
    End If
    
    ' Get the property
    IPS.ReadMultiple 1, PROP, Value

    ' Check Value type and convert if necesary
    If Not IsEmpty(Value) Then
        
        VT = VarType(Value)
        
        Select Case VT
            
            Case VT_LPSTR, VT_LPWSTR
                GetPropertyByName = ToBSTR(Value)
                        
            Case VT_FILETIME
                GetPropertyByName = ToDate(Value)
            
            Case VT_BOOL
                GetPropertyByName = Value
                            
            Case VT_CF
            
            Case Else
            
                If IsNumeric(Value) Then
                    
                    GetPropertyByName = CLng(Value)
                
                Else
                    
                    If (VT And VT_VECTOR) Then
                        
                        If (VT And VT_LPSTR) Then
                            
                            GetPropertyByName = ToBSTRArray(Value)
                        
                        ElseIf (VT And VT_LPWSTR) Then
                            
                            GetPropertyByName = ToBSTRArray(Value, True)
                            
                        End If
                    
                    End If
                    
                End If
                
        End Select
        
    End If
    
    ' Clear Value
    PropVariantClear Value
    Set IPS = Nothing
    
End Function


Public Sub SetPropertyByName(ByVal PropSet As odpPropertySets, ByVal Name As Variant, ByVal NewValue As Variant)
Dim PROP As PROPSPEC, Value As Variant, PropName As String
Dim IPS As IPropertyStorage, VT As Integer, Clsid As IID

    If m_PropSet Is Nothing Then Err.Raise odpFileIsClosed, , odpFileIsCloseStr
        
    Select Case PropSet
        
        Case odpSummary
            
            Set IPS = m_PropSet.Open(FMTID_SummaryInformation, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
        
        Case odpDocSummary
            
            Set IPS = m_PropSet.Open(FMTID_DocSummaryInformation, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
        
        Case odpUserProperties
            
            On Error Resume Next

            Set IPS = m_PropSet.Open(FMTID_UserProperties, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
            
            If Err.Number <> 0 Then
                Set IPS = m_PropSet.create(FMTID_UserProperties, Clsid, PROPSETFLAG_ANSI, STGM_READWRITE Or STGM_SHARE_EXCLUSIVE)
                m_Storage.Commit STGC_DEFAULT
            End If
            
            On Error GoTo 0
    
    End Select

    If IsNumeric(Name) Then
        
        PROP.ulKind = PRSPEC_PROPID
        PROP.ID_or_LPWSTR = CLng(Name)
    
    Else
        
        PropName = CStr(Name)
        
        PROP.ulKind = PRSPEC_LPWSTR
        PROP.ID_or_LPWSTR = StrPtr(PropName)
        
    End If

    VT = VarType(NewValue)
    
    Select Case VT
        
        Case VT_BSTR
            ToLPSTR NewValue, Value
        
        Case VT_DATE
            ToFILETIME NewValue, Value
        
        Case VT_BOOL
            Value = NewValue
        
        Case Else
            If VT = VT_ARRAY Or VT_BSTR Then
                ToLPSTRArray NewValue, Value
            Else
                Value = CLng(NewValue)
            End If
            
    End Select
    
    ' Write the property
    IPS.WriteMultiple 1, PROP, Value
    
    ' Save property storage
    IPS.Commit STGC_DEFAULT
    
    ' Clear Value
    PropVariantClear Value
    Set IPS = Nothing
    
End Sub



Public Sub Revert()

    m_Storage.Revert
    
End Sub

Public Sub Save()

    m_Storage.Commit STGC_DEFAULT
    
End Sub


'*********************************************************************************************
' Returns the document Title
'*********************************************************************************************
Public Property Get Title() As String

    Title = GetPropertyByName(odpSummary, PIDSI_TITLE)
    
End Property

Public Property Let Title(ByVal New_Value As String)

    SetPropertyByName odpSummary, PIDSI_TITLE, New_Value
    
End Property


Public Sub BindToFile(ByVal FileName As String)
Dim res As Long

    ' Check if the file is a storage file
    res = StgIsStorageFile(FileName)
    
    If res <> 0 Then
        If (Err And &H80000000) Then
            Err.Raise res
        Else
            Err.Raise odpInvalidFile, , FileName & " is not a structured storage file."
        End If
    End If
    
    ' Open the storage file
    Set m_Storage = StgOpenStorage(FileName, , STGM_READWRITE Or STGM_TRANSACTED Or STGM_SHARE_DENY_WRITE)
    
    ' Get IPropertySetStorage
    Set m_PropSet = m_Storage
        
End Sub

Public Sub BindToStorage(ByVal Stg As Object)

    ' Open the storage file
    Set m_Storage = Stg.Storage
    
    ' Get IPropertySetStorage
    Set m_PropSet = m_Storage
        
End Sub


Public Property Get Category() As String

    Category = GetPropertyByName(odpDocSummary, PID_CATEGORY)
    
End Property

Public Property Let Category(ByVal New_Value As String)

    SetPropertyByName odpDocSummary, PID_CATEGORY, New_Value
    
End Property
Public Property Get Company() As String

    Company = GetPropertyByName(odpDocSummary, PID_COMPANY)
    
End Property

Public Property Let Company(ByVal NewValue As String)

    SetPropertyByName odpDocSummary, PID_COMPANY, NewValue
    
End Property


Public Property Get PresentationTarget() As String

    PresentationTarget = GetPropertyByName(odpDocSummary, PID_PRESFORMAT)
    
End Property

Public Property Let PresentationTarget(ByVal NewValue As String)

    SetPropertyByName odpDocSummary, PID_PRESFORMAT, NewValue
    
End Property


Public Property Get Bytes() As Long

    Bytes = GetPropertyByName(odpDocSummary, PID_BYTECOUNT)
    
End Property

Public Property Let Bytes(ByVal NewValue As Long)

    SetPropertyByName odpDocSummary, PID_BYTECOUNT, NewValue
    
End Property


Public Property Get Lines() As Long

    Lines = GetPropertyByName(odpDocSummary, PID_LINECOUNT)
    
End Property

Public Property Let Lines(ByVal NewValue As Long)

    SetPropertyByName odpDocSummary, PID_LINECOUNT, NewValue
    
End Property


Public Property Get Paragraphs() As Long

    Paragraphs = GetPropertyByName(odpDocSummary, PID_PARCOUNT)
    
End Property

Public Property Let Paragraphs(ByVal NewValue As Long)

    SetPropertyByName odpDocSummary, PID_PARCOUNT, NewValue
    
End Property


Public Property Get Slides() As Long

    Slides = GetPropertyByName(odpDocSummary, PID_SLIDECOUNT)
    
End Property

Public Property Let Slides(ByVal NewValue As Long)

    SetPropertyByName odpDocSummary, PID_SLIDECOUNT, NewValue
    
End Property


Public Property Get Notes() As Long

    Notes = GetPropertyByName(odpDocSummary, PID_NOTECOUNT)
    
End Property

Public Property Let Notes(ByVal NewValue As Long)

    SetPropertyByName odpDocSummary, PID_NOTECOUNT, NewValue
    
End Property


Public Property Get HiddenSlides() As Long

    HiddenSlides = GetPropertyByName(odpDocSummary, PID_HIDDENCOUNT)
    
End Property

Public Property Let HiddenSlides(ByVal NewValue As Long)

    SetPropertyByName odpDocSummary, PID_HIDDENCOUNT, NewValue
    
End Property


Public Property Get MMClips() As Long

    MMClips = GetPropertyByName(odpDocSummary, PID_MMCLIPCOUNT)
    
End Property

Public Property Let MMClips(ByVal NewValue As Long)

    SetPropertyByName odpDocSummary, PID_MMCLIPCOUNT, NewValue
    
End Property


Private Sub Class_Initialize()

    ' Initialize FMTIDs
    If Not FMTID_Init Then InitFMTIDs
    
End Sub

Public Property Get Application() As String

    Application = GetPropertyByName(odpSummary, PIDSI_APPNAME)
    
End Property

Public Property Let Application(ByVal New_Value As String)

    SetPropertyByName odpSummary, PIDSI_APPNAME, New_Value
    
End Property


Public Property Get Chars() As Long

    Chars = GetPropertyByName(odpSummary, PIDSI_CHARCOUNT)
    
End Property

Public Property Let Chars(ByVal NewValue As Long)

    SetPropertyByName odpSummary, PIDSI_CHARCOUNT, NewValue
    
End Property


Public Property Get Comments() As String
    
    Comments = GetPropertyByName(odpSummary, PIDSI_COMMENTS)

End Property

Public Property Let Comments(ByVal New_Value As String)

    SetPropertyByName odpSummary, PIDSI_COMMENTS, New_Value
    
End Property

Public Property Get CreateDate() As Date

    CreateDate = GetPropertyByName(odpSummary, PIDSI_CREATE_DTM)
    
End Property

Public Property Let CreateDate(ByVal NewValue As Date)

    SetPropertyByName odpSummary, PIDSI_CREATE_DTM, NewValue
    
End Property
Public Property Get EditingTime() As Date

    EditingTime = GetPropertyByName(odpSummary, PIDSI_EDITTIME)

End Property

Public Property Let EditingTime(ByVal NewValue As Date)

    SetPropertyByName odpSummary, PIDSI_EDITTIME, NewValue

End Property


Public Property Get Keywords() As String
    
    Keywords = GetPropertyByName(odpSummary, PIDSI_KEYWORDS)
    
End Property

Public Property Let Keywords(ByVal New_Value As String)
    
    SetPropertyByName odpSummary, PIDSI_KEYWORDS, New_Value

End Property

Public Property Get LastSavedBy() As String

    LastSavedBy = GetPropertyByName(odpSummary, PIDSI_LASTAUTHOR)
    
End Property

Public Property Let LastSavedBy(ByVal New_Value As String)

    SetPropertyByName odpSummary, PIDSI_LASTAUTHOR, New_Value
    
End Property


Public Property Get PrintDate() As Date
    
    PrintDate = GetPropertyByName(odpSummary, PIDSI_LASTPRINTED)

End Property

Public Property Let PrintDate(ByVal NewValue As Date)
    
    SetPropertyByName odpSummary, PIDSI_LASTPRINTED, NewValue

End Property


Public Property Get LastSaveDate() As Date
    
    LastSaveDate = GetPropertyByName(odpSummary, PIDSI_LASTSAVE_DTM)

End Property

Public Property Let LastSaveDate(ByVal NewValue As Date)
    
    SetPropertyByName odpSummary, PIDSI_LASTSAVE_DTM, NewValue

End Property


Public Property Get Subject() As String
    
    Subject = GetPropertyByName(odpSummary, PIDSI_SUBJECT)

End Property

Public Property Let Subject(ByVal New_Value As String)
    
    SetPropertyByName odpSummary, PIDSI_SUBJECT, New_Value

End Property

Public Property Get Template() As String

    Template = GetPropertyByName(odpSummary, PIDSI_TEMPLATE)
    
End Property

Public Property Let Template(ByVal NewValue As String)

    SetPropertyByName odpSummary, PIDSI_TEMPLATE, NewValue
    
End Property


Public Property Get Pages() As Long
    
    Pages = GetPropertyByName(odpSummary, PIDSI_PAGECOUNT)

End Property

Public Property Let Pages(ByVal NewValue As Long)
    
    SetPropertyByName odpSummary, PIDSI_PAGECOUNT, NewValue

End Property


Public Property Get Words() As Long
    
    Words = GetPropertyByName(odpSummary, PIDSI_WORDCOUNT)

End Property

Public Property Let Words(ByVal NewValue As Long)
    
    SetPropertyByName odpSummary, PIDSI_WORDCOUNT, NewValue

End Property


Public Property Get Revision() As String
    
    Revision = GetPropertyByName(odpSummary, PIDSI_REVNUMBER)

End Property


Public Property Let Revision(ByVal NewValue As String)
    
    SetPropertyByName odpSummary, PIDSI_REVNUMBER, NewValue

End Property



